unit LstItemColor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm26 = class(TForm)
    ListBox1: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  Form26: TForm26;

implementation

{$R *.dfm}


type
  TItemColor = class(TInterfacedObject)
  private
    // couleur de la police de l'item
    fFntCol : TColor;
    // couleur du fond de l'item
    fGndCol : TColor;
    // style de la police de l'item
    fFntStl : TFontStyles;
    // listbox d'origine de l'item
    fOwner  : TListBox;
    procedure SetColor(const Index: Integer; const Value: TColor);
    procedure SetFntStl(const Value: TFontStyles);
  published
    property FontColor   : TColor index 0 read fFntCol write SetColor default clBlack;
    property GroundColor : TColor index 1 read fGndCol write SetColor default clWhite;
    property FontStyle   : TFontStyles    read fFntStl write SetFntStl;
  public
    constructor Create(const aOwner: TListBox; const aFontColor,
                       aGroundColor: TColor; const aFontStyle: TFontStyles = []);
    destructor Destroy; override;
  end;

var
  __Garbage : TList;


{ TItemColor }

constructor TItemColor.Create(const aOwner: TListBox; const aFontColor,
                              aGroundColor: TColor; const aFontStyle: TFontStyles = []);
begin
  fFntCol := aFontColor;
  fGndCol := aGroundColor;
  fFntStl := aFontStyle;
  fOwner  := aOwner;
  __Garbage.Add(pointer(Self));
end;

destructor TItemColor.Destroy;
var I : integer;
begin
  I := __Garbage.IndexOf(pointer(Self));
  if I <> -1 then
    __Garbage.Delete(I);
  inherited;
end;

procedure TItemColor.SetColor(const Index: Integer; const Value: TColor);
var ptr : ^TColor;
begin
  case index of
    0 : ptr := @fFntCol;
    1 : ptr := @fGndCol;
  end;
  if ptr^ <> Value then
  begin
    ptr^ := Value;
    if assigned(fOwner) then
      fOwner.Invalidate;
  end;
end;

procedure TItemColor.SetFntStl(const Value: TFontStyles);
begin
  if fFntStl <> Value then
  begin
    fFntStl := Value;
    if assigned(fOwner) then
      fOwner.Invalidate;
  end;
end;



{ fiche }

procedure TForm26.FormCreate(Sender: TObject);
begin
  ListBox1.Align := alClient;
  ListBox1.Style := lbOwnerDrawFixed;

  ListBox1.Items.Add('Alpha');
  ListBox1.Items.AddObject('Beta', TItemColor.Create(ListBox1, $0099FF, $9C7454, [fsBold]));
  ListBox1.Items.Add('Gamma');
  ListBox1.Items.AddObject('Delta', TItemColor.Create(ListBox1, clYellow, clNavy, [fsItalic]));
  ListBox1.Items.Add('Epsilon');

  ListBox1.Items.Objects[2] := TItemColor.Create(ListBox1, clWhite, clRed);
end;

procedure TForm26.ListBox1DrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  LB : TListBox;
  IC : TItemColor;
  T  : TColor;
  R  : TRect;
begin
  LB := (control as TListBox);
  IC   := TItemColor(LB.Items.Objects[index]);

  if assigned(IC) then
  begin
    LB.Canvas.Brush.Color := IC.GroundColor;
    LB.Canvas.Font.Color  := IC.FontColor;
    LB.Canvas.Font.Style  := IC.FontStyle;
  end
  else
  begin
    LB.Canvas.Brush.Color := clWhite;
    LB.Canvas.Font.Color  := clBlack;
    LB.Canvas.Font.Style  := [];
  end;

  if (odSelected in State) or (odFocused in State) then
  begin
    T := LB.Canvas.Brush.Color;
    LB.Canvas.Brush.Color := LB.Canvas.Font.Color;
    LB.Canvas.Font.Color  := T;
  end;

  LB.Canvas.FillRect(Rect);
  R := Rect;
  inc(R.Left, 2);
  DrawText(LB.Canvas.Handle, LB.Items[Index], -1, R, DT_LEFT or DT_VCENTER or DT_SINGLELINE);
end;



initialization
  __Garbage := TList.Create;

finalization
  while __Garbage.Count > 0 do
    TItemColor(__Garbage.Items[__Garbage.Count-1]).Free;
  __Garbage.Free;

end.
